@@ -8,6 +8,7 @@ import os, glob
 import pandas as pd
 import numpy as np
 from itertools import product
+from numba import jit, prange
 
 class UserDefinedFeatureExtractor(object):
     """
@@ -50,6 +51,16 @@ class UserDefinedFeatureExtractor(object):
         self.files_found = glob.glob(self.in_folder + '/*.' + self.file_type)
         print('Extracting features from {} file(s)...'.format(str(len(self.files_found))))
 
+    @staticmethod
+    @jit(nopython=True)
+    def __count_values_in_range(data: np.array, ranges: np.array):
+        results = np.full((data.shape[0], ranges.shape[0]), 0)
+        for i in prange(data.shape[0]):
+            for j in prange(ranges.shape[0]):
+                lower_bound, upper_bound = ranges[j][0], ranges[j][1]
+                results[i][j] = data[i][np.logical_and(data[i] >= lower_bound, data[i] <= upper_bound)].shape[0]
+        return results
+
     def __euclid_dist_between_bps_of_other_animals(self):
         print('Calculating euclidean distances...')
         self.distance_col_names = []
@@ -98,15 +109,12 @@ class UserDefinedFeatureExtractor(object):
             self.data_df['Sum_{}_{}'.format(i[1], i[0])] = self.data_df[i[1]].rolling(int(i[0]), min_periods=1).sum()
 
     def __pose_probability_filters(self):
-        def count_values_in_range(series, values_in_range_min, values_in_range_max):
-            return series.between(left=values_in_range_min, right=values_in_range_max).sum()
-
         p_df = self.data_df.filter(self.pcols, axis=1)
         self.data_df['Sum_probabilities'] = p_df.sum(axis=1)
         self.data_df['Mean_probabilities'] = p_df.mean(axis=1)
-        p_ranges_dict = {"0.1": [0.000000000, 0.1], "0.5": [0.000000000, 0.5], "0.75": [0.000000000, 0.75]}
-        for name, p_range in p_ranges_dict.items():
-            self.data_df["Low_prob_detections_" + name] = p_df.apply(func=lambda row: count_values_in_range(row, p_range[0], p_range[1]), axis=1)
+
+        results = pd.DataFrame(self.__count_values_in_range(data=self.out_data.filter(self.pcols).values, ranges=np.array([[0.0, 0.1], [0.000000000, 0.5], [0.000000000, 0.75]])), columns=['Low_prob_detections_0.1', 'Low_prob_detections_0.5','Low_prob_detections_0.75'])
+        self.out_data = pd.concat([self.data_df, results], axis=1)
 
     def extract_features(self):
         """
