@@ -187,6 +187,7 @@ class AdvancedLabellingInterface(object):
                                  '\n Left Arrow = -1 frame'
                                  '\n Ctrl + s = Save annotations file'
                                  '\n Ctrl + a = +1 frame and keep choices'
+                                 '\n Ctrl + p = Show annotation statistics'
                                  '\n Ctrl + l = Last frame'
                                  '\n Ctrl + o = First frame')
         self.key_presses_lbl.grid(sticky=S)
@@ -196,11 +197,25 @@ class AdvancedLabellingInterface(object):
     def bind_shortcut_keys(self):
         self.main_window.bind('<Control-s>', lambda x: self.save_results())
         self.main_window.bind('<Control-a>', lambda x: self.advance_frame(new_frm_number= int(self.current_frm_n.get() + 1), keep_prior_img_cb_status=True))
+        self.main_window.bind('<Control-p>', lambda x: self.__print_annotation_statistics())
         self.main_window.bind('<Right>', lambda x: self.advance_frame(new_frm_number= int(self.current_frm_n.get() + 1)))
         self.main_window.bind('<Left>', lambda x: self.advance_frame(new_frm_number= int(self.current_frm_n.get() - 1)))
         self.main_window.bind('<Control-l>', lambda x: self.advance_frame(new_frm_number = self.max_frm_no))
         self.main_window.bind('<Control-o>', lambda x: self.advance_frame(0))
 
+
+    def __print_annotation_statistics(self):
+        table_view = [["Video name", self.video_name], ['Video frames', self.video_meta_data['frame_count']]]
+        for target in self.target_lst:
+            present = sum(self.data_df_targets[target][self.data_df_targets[target] == 1])
+            absent = sum(self.data_df_targets[target][self.data_df_targets[target] == 0])
+            table_view.append([target + ' present labels', present])
+            table_view.append([target + ' absent labels', absent])
+            table_view.append([target + ' % present', present / self.video_meta_data['frame_count']])
+            table_view.append([target + ' % absent', absent / self.video_meta_data['frame_count']])
+        headers = ["VARIABLE", "VALUE"]
+        print(tabulate(table_view, headers, tablefmt="github"))
+
     def play_video(self):
         p = Popen('python {}'.format(self.play_video_script_path), stdin=PIPE, stdout=PIPE, shell=True)
         main_project_dir = os.path.dirname(self.config_path)
