@@ -16,9 +16,9 @@ from tkinter import *
 from tkinter import filedialog
 from PIL import Image, ImageTk
 from subprocess import Popen, PIPE
-from copy import deepcopy
 import os
 from simba.train_model_functions import get_all_clf_names
+from tabulate import tabulate
 
 class LabellingInterface(object):
     """
@@ -193,6 +193,7 @@ class LabellingInterface(object):
                                  '\n Left Arrow = -1 frame'
                                  '\n Ctrl + s = Save annotations file'
                                  '\n Ctrl + a = +1 frame and keep choices'
+                                 '\n Ctrl + p = Show annotation statistics'
                                  '\n Ctrl + l = Last frame'
                                  '\n Ctrl + o = First frame')
         self.key_presses_lbl.grid(sticky=S)
@@ -202,11 +203,24 @@ class LabellingInterface(object):
     def __bind_shortcut_keys(self):
         self.main_window.bind('<Control-s>', lambda x: self.__save_results())
         self.main_window.bind('<Control-a>', lambda x: self.__advance_frame(new_frm_number=int(self.current_frm_n.get() + 1), save_and_keep_checks=True))
+        self.main_window.bind('<Control-p>', lambda x: self.print_annotation_statistics())
         self.main_window.bind('<Right>', lambda x: self.__advance_frame(new_frm_number= int(self.current_frm_n.get() + 1)))
         self.main_window.bind('<Left>', lambda x: self.__advance_frame(new_frm_number= int(self.current_frm_n.get() - 1)))
         self.main_window.bind('<Control-l>', lambda x: self.__advance_frame(new_frm_number = self.max_frm_no))
         self.main_window.bind('<Control-o>', lambda x: self.__advance_frame(0))
 
+    def print_annotation_statistics(self):
+        table_view = [["Video name", self.video_name], ['Video frames', self.video_meta_data['frame_count']]]
+        for target in self.target_lst:
+            present = len(self.data_df_targets[self.data_df_targets[target] == 1])
+            absent = len(self.data_df_targets[self.data_df_targets[target] == 0])
+            table_view.append([target + ' present labels', present])
+            table_view.append([target + ' absent labels', absent])
+            table_view.append([target + ' % present', present / self.video_meta_data['frame_count']])
+            table_view.append([target + ' % absent', absent / self.video_meta_data['frame_count']])
+        headers = ["VARIABLE", "VALUE"]
+        print(tabulate(table_view, headers, tablefmt="github"))
+
     def __play_video(self):
         p = Popen('python {}'.format(self.play_video_script_path), stdin=PIPE, stdout=PIPE, shell=True)
         main_project_dir = os.path.dirname(self.config_path)
@@ -268,7 +282,7 @@ class LabellingInterface(object):
             check_int('END FRAME', int(end_frm), max_value=self.max_frm_no, min_value=0)
             for frm_no in range(int(start_frm), int(end_frm) + 1):
                 for target in self.target_lst:
-                    self.data_df_targets[target].loc[frm_no] = self.checkboxes[target]['var'][target].get()
+                    self.data_df_targets[target].loc[frm_no] = self.checkboxes[target]['var'].get()
             self.__read_frm(frm_number=int(end_frm))
             self.change_frm_box.delete(0, END)
             self.change_frm_box.insert(0, end_frm)
@@ -341,5 +355,5 @@ def select_labelling_video(config_path: str=None,
 
 # test = select_labelling_video(config_path='/Users/simon/Desktop/troubleshooting/train_model_project/project_folder/project_config.ini',
 #                           threshold_dict={'Attack': 0.4},
-#                           setting='pseudo',
+#                           setting='from_scratch',
 #                           continuing=False)
