@@ -304,19 +304,21 @@ def find_video_of_file(video_dir: str,
     return_path: str
 
     """
-
-    all_files_in_video_folder = [f for f in next(os.walk(video_dir))[2] if not f[0] == '.']
+    try:
+        all_files_in_video_folder = [f for f in next(os.walk(video_dir))[2] if not f[0] == '.']
+    except StopIteration:
+        print('SIMBA ERROR: No files found in the {} directory'.format(video_dir))
+        raise FileNotFoundError('SIMBA ERROR: No files found in the {} directory'.format(video_dir))
     all_files_in_video_folder = [os.path.join(video_dir, x) for x in all_files_in_video_folder]
-    return_path = ''
+    return_path = None
     for file_path in all_files_in_video_folder:
         _, video_filename, ext = get_fn_ext(file_path)
         if ((video_filename == filename) and ((ext.lower() == '.mp4') or (ext.lower() == '.avi'))):
             return_path = file_path
 
-    if return_path == '':
-        print('FAILED: SimBA could not find a video file resenting {} in the project_folder/videos directory'.format(str(filename)))
-    else:
-        return return_path
+    if return_path is None:
+        print('SIMBA WARNING: SimBA could not find a video file resenting {} in the project video directory'.format(str(filename)))
+    return return_path
 
 
 def smooth_data_gaussian(config: configparser.ConfigParser,
