@@ -6,29 +6,32 @@ from simba.read_config_unit_tests import (read_config_file,
 import multiprocessing
 import webbrowser
 from tkinter import *
+import simba
 from simba.misc_tools import (check_multi_animal_status,
                               get_video_meta_data,
                               convert_parquet_to_csv,
                               convert_csv_to_parquet,
-                              tabulate_clf_info)
-from simba.drop_bp_cords import getBpNames, create_body_part_dictionary
+                              tabulate_clf_info,
+                              find_video_of_file)
+from simba.drop_bp_cords import getBpNames, create_body_part_dictionary, get_fn_ext
 from simba.get_coordinates_tools_v2 import get_coordinates_nilsson
 from simba.heat_mapper_location import HeatmapperLocation
-import os, glob
-from simba.ez_lineplot import draw_line_plot
 from simba.classifications_per_ROI import clf_within_ROI
 from simba.tkinter_functions import hxtScrollbar, DropDownMenu, CreateToolTip
-from collections import defaultdict
 from simba.train_model_functions import get_all_clf_names
 from simba.tkinter_functions import Entry_Box, FileSelect, FolderSelect
 from simba.reorganize_keypoint_in_pose import KeypointReorganizer
-from simba.ez_lineplot import draw_line_plot,draw_line_plot_tools
+from simba.ez_lineplot import DrawPathPlot
 from simba.FSTTC_calculator import FSTTCPerformer
 from simba.Kleinberg_calculator import KleinbergCalculator
 from simba.timebins_clf_analyzer import TimeBinsClf
 from simba.create_clf_log import ClfLogCreator
+from simba.ez_lineplot import draw_line_plot
 from simba.multi_cropper import MultiCropper
+from simba.remove_keypoints_in_pose import KeypointRemover
+from simba.plot_pose_in_dir import create_video_from_dir
 from simba.extract_seqframes import extract_seq_frames
+from simba.frame_mergerer_ffmpeg import FrameMergererFFmpeg
 from simba.video_processing import (downsample_video,
                                     clahe_enhance_video,
                                     crop_single_video,
@@ -46,7 +49,11 @@ from simba.video_processing import (downsample_video,
                                     change_single_video_fps,
                                     change_fps_of_multiple_videos,
                                     frames_to_movie,
-                                    gif_creator)
+                                    gif_creator,
+                                    video_concatenator)
+from collections import defaultdict
+from PIL import Image, ImageTk
+import os, glob
 
 class HeatmapLocationPopup(object):
     def __init__(self,
@@ -61,7 +68,7 @@ class HeatmapLocationPopup(object):
         self.project_animal_cnt = read_config_entry(config=self.config, section='General settings', option='animal_no', data_type='int')
         self.multi_animal_status, self.multi_animal_id_lst = check_multi_animal_status(self.config, self.project_animal_cnt)
         self.x_cols, self.y_cols, self.pcols = getBpNames(config_path)
-        self.animal_bp_dict = create_body_part_dictionary(self.multi_animal_status, self.multi_animal_id_lst, self.project_animal_cnt, self.x_cols, self.y_cols, [], [])
+        self.animal_bp_dict = create_body_part_dictionary(self.multi_animal_status, list(self.multi_animal_id_lst), self.project_animal_cnt, self.x_cols, self.y_cols, [], [])
         self.all_body_parts = []
         for animal, bp_cords in self.animal_bp_dict.items():
             for bp_dim, bp_data in bp_cords.items(): self.all_body_parts.extend(([x[:-2] for x in bp_data]))
@@ -164,63 +171,85 @@ class QuickLineplotPopup(object):
         run_btn =  Button(self.settings_frm,text='Create path plot',command=lambda: draw_line_plot(self.config_path, self.chosen_video_val.get(), self.chosen_bp_val.get()))
         run_btn.grid(row=2, column=1, pady=10)
 
-class ClfByROI(object):
+class ClfByROIPopUp(object):
     def __init__(self,
                  config_path: str):
         ROI_clf_toplevel = Toplevel()
-        ROI_clf_toplevel.minsize(400, 400)
+        ROI_clf_toplevel.minsize(400, 700)
         ROI_clf_toplevel.wm_title("Classifications by ROI settings")
         ROI_clf_toplevel.lift()
         ROI_clf_toplevel = Canvas(hxtScrollbar(ROI_clf_toplevel))
         ROI_clf_toplevel.pack(fill="both", expand=True)
 
-        ROI_menu = LabelFrame(ROI_clf_toplevel, text='Select_ROI(s)', padx=5, pady=5)
+        ROI_menu = LabelFrame(ROI_clf_toplevel, text='Select ROI(s)', padx=5, pady=5)
         classifier_menu = LabelFrame(ROI_clf_toplevel, text='Select classifier(s)', padx=5, pady=5)
         body_part_menu = LabelFrame(ROI_clf_toplevel, text='Select body part', padx=5, pady=5)
-        self.menu_items = clf_within_ROI(config_path)
+        measurements_menu = LabelFrame(ROI_clf_toplevel, text='Select measurements', padx=5, pady=5)
+        self.clf_roi_analyzer = clf_within_ROI(config_path)
+        self.total_time_var = BooleanVar()
+        self.start_bouts_var = BooleanVar()
+        self.end_bouts_var = BooleanVar()
+        self.total_time_cb = Checkbutton(measurements_menu, text='Total time by ROI (s)', variable=self.total_time_var)
+        self.start_bouts_cb = Checkbutton(measurements_menu, text='Started bouts by ROI (count)', variable=self.start_bouts_var)
+        self.end_bouts_cb = Checkbutton(measurements_menu, text='Ended bouts by ROI (count)', variable=self.end_bouts_var)
         self.ROI_check_boxes_status_dict = {}
         self.clf_check_boxes_status_dict = {}
 
-        for row_number, ROI in enumerate(self.menu_items.ROI_str_name_list):
+        for row_number, ROI in enumerate(self.clf_roi_analyzer.ROI_str_name_list):
             self.ROI_check_boxes_status_dict[ROI] = IntVar()
             ROI_check_button = Checkbutton(ROI_menu, text=ROI, variable=self.ROI_check_boxes_status_dict[ROI])
             ROI_check_button.grid(row=row_number, sticky=W)
 
-        for row_number, clf_name in enumerate(self.menu_items.behavior_names):
+        for row_number, clf_name in enumerate(self.clf_roi_analyzer.behavior_names):
             self.clf_check_boxes_status_dict[clf_name] = IntVar()
             clf_check_button = Checkbutton(classifier_menu, text=clf_name,
                                            variable=self.clf_check_boxes_status_dict[clf_name])
             clf_check_button.grid(row=row_number, sticky=W)
 
-        self.choose_bp = DropDownMenu(body_part_menu, 'Body part', self.menu_items.body_part_list, '12')
-        self.choose_bp.setChoices(self.menu_items.body_part_list[0])
+        self.choose_bp = DropDownMenu(body_part_menu, 'Body part', self.clf_roi_analyzer.body_part_list, '12')
+        self.choose_bp.setChoices(self.clf_roi_analyzer.body_part_list[0])
         self.choose_bp.grid(row=0, sticky=W)
-        run_analysis_button = Button(ROI_clf_toplevel, text='Analyze classifications in each ROI',
-                                     command=lambda: self.run_clf_by_ROI_analysis())
+        run_analysis_button = Button(ROI_clf_toplevel, text='Analyze classifications in each ROI',command=lambda: self.run_clf_by_ROI_analysis())
         body_part_menu.grid(row=0, sticky=W, padx=10, pady=10)
         ROI_menu.grid(row=1, sticky=W, padx=10, pady=10)
         classifier_menu.grid(row=2, sticky=W, padx=10, pady=10)
-        run_analysis_button.grid(row=3, sticky=W, padx=10, pady=10)
+        self.total_time_cb.grid(row=0, sticky=NW)
+        self.start_bouts_cb.grid(row=1, sticky=NW)
+        self.end_bouts_cb.grid(row=2, sticky=NW)
+        measurements_menu.grid(row=3, sticky=W, padx=10, pady=10)
+        run_analysis_button.grid(row=4, sticky=W, padx=10, pady=10)
 
     def run_clf_by_ROI_analysis(self):
         body_part_list = [self.choose_bp.getChoices()]
         ROI_dict_lists, behavior_list = defaultdict(list), []
+        measurements_list = []
         for loop_val, ROI_entry in enumerate(self.ROI_check_boxes_status_dict):
             check_val = self.ROI_check_boxes_status_dict[ROI_entry]
             if check_val.get() == 1:
-                shape_type = self.menu_items.ROI_str_name_list[loop_val].split(':')[0].replace(':', '')
-                shape_name = self.menu_items.ROI_str_name_list[loop_val].split(':')[1].replace(' ', '')
+                shape_type = self.clf_roi_analyzer.ROI_str_name_list[loop_val].split(':')[0].replace(':', '')
+                shape_name = self.clf_roi_analyzer.ROI_str_name_list[loop_val].split(':')[1][1:]
                 ROI_dict_lists[shape_type].append(shape_name)
 
+        for measurement_var, measurement_name in zip([self.total_time_var.get(), self.start_bouts_var.get(), self.end_bouts_var.get()], ['Total time by ROI (s)', 'Started bouts by ROI (count)', 'Ended bouts by ROI (count)']):
+            if measurement_var:
+                measurements_list.append(measurement_name)
+
+
         for loop_val, clf_entry in enumerate(self.clf_check_boxes_status_dict):
             check_val = self.clf_check_boxes_status_dict[clf_entry]
             if check_val.get() == 1:
-                behavior_list.append(self.menu_items.behavior_names[loop_val])
-        if len(ROI_dict_lists) == 0: print('No ROIs selected.')
-        if len(behavior_list) == 0: print('No classifiers selected.')
-
+                behavior_list.append(self.clf_roi_analyzer.behavior_names[loop_val])
+        if len(ROI_dict_lists) == 0:
+            print('SIMBA ERROR: Please select at least one ROI.')
+            raise ValueError('SIMBA ERROR: Please select at least one ROI.')
+        if len(behavior_list) == 0:
+            print('SIMBA ERROR: Please select at least one behavior classifier.')
+            raise ValueError('SIMBA ERROR: Please select at least one behavior classifier.')
+        if len(measurements_list) == 0:
+            print('SIMBA ERROR: Please select at least one measurement.')
+            raise ValueError('SIMBA ERROR: Please select at least one measurement.')
         else:
-            clf_within_ROI.perform_ROI_clf_analysis(self.menu_items, ROI_dict_lists, behavior_list, body_part_list)
+            self.clf_roi_analyzer.perform_ROI_clf_analysis(ROI_dict_lists=ROI_dict_lists, behavior_list=behavior_list, body_part_list=body_part_list, measurements=measurements_list)
 
 
 class FSTTCPopUp(object):
@@ -917,9 +946,9 @@ class MakePathPlotPopUp(object):
         main_frm.minsize(200, 200)
         main_frm.wm_title("CREATE PATH PLOT")
         settings_frm = LabelFrame(main_frm)
-        video_path = FileSelect(settings_frm, 'VIDEO PATH: ', lblwidth='10')
-        body_part = Entry_Box(settings_frm, 'BODY PART: ', '10')
-        csv_path = FileSelect(settings_frm, 'CSV PATH: ', lblwidth='10')
+        video_path = FileSelect(settings_frm, 'VIDEO PATH: ', lblwidth='30')
+        body_part = Entry_Box(settings_frm, 'BODY PART: ', '30')
+        data_path = FileSelect(settings_frm, 'DATA PATH (e.g., DLC H5 or CSV file): ', lblwidth='30')
         color_lst = ['White',
                      'Black',
                      'Grey',
@@ -939,17 +968,29 @@ class MakePathPlotPopUp(object):
                      'Purple',
                      'Cyan']
         background_color = DropDownMenu(settings_frm,'BACKGROUND COLOR: ',color_lst,'10')
+        background_color.setChoices(choice='White')
         line_color = DropDownMenu(settings_frm, 'LINE COLOR: ', color_lst, '10')
+        line_color.setChoices(choice='Red')
         line_thickness = DropDownMenu(settings_frm, 'LINE THICKNESS: ', list(range(1, 11)), '10')
-        run_btn = Button(settings_frm,text='CREATE PATH PLOT VIDEO',command =lambda: draw_line_plot_tools(video_path.file_path, csv_path.file_path, body_part.entry_get))
+        line_thickness.setChoices(choice=1)
+        circle_size = DropDownMenu(settings_frm, 'CIRCLE SIZE: ', list(range(1, 11)), '10')
+        circle_size.setChoices(choice=5)
+        run_btn = Button(settings_frm,text='CREATE PATH PLOT VIDEO',command = lambda: DrawPathPlot(data_path=data_path.file_path,
+                                                                                                                video_path=video_path.file_path,
+                                                                                                                body_part=body_part.entry_get,
+                                                                                                                bg_color=background_color.getChoices(),
+                                                                                                                line_color=line_color.getChoices(),
+                                                                                                                line_thinkness=line_thickness.getChoices(),
+                                                                                                                circle_size=circle_size.getChoices()))
         settings_frm.grid(row=0,sticky=W)
         video_path.grid(row=0,sticky=W)
-        csv_path.grid(row=1,sticky=W)
+        data_path.grid(row=1,sticky=W)
         body_part.grid(row=2,sticky=W)
         background_color.grid(row=3,sticky=W)
         line_color.grid(row=4, sticky=W)
         line_thickness.grid(row=5, sticky=W)
-        run_btn.grid(row=6,pady=10)
+        circle_size.grid(row=6, sticky=W)
+        run_btn.grid(row=7,pady=10)
 
 class PoseReorganizerPopUp(object):
     def __init__(self):
@@ -1034,4 +1075,416 @@ class PoseReorganizerPopUp(object):
             for curr_animal, curr_bp in zip(self.newanimallist, self.newbplist):
                 new_bp_list.append(curr_bp.getChoices())
                 new_animal_list.append(curr_animal.getChoices())
-            self.keypoint_reorganizer.perform_reorganization(animal_list=new_animal_list, bp_lst=new_bp_list)
\ No newline at end of file
+            self.keypoint_reorganizer.perform_reorganization(animal_list=new_animal_list, bp_lst=new_bp_list)
+
+class AboutSimBAPopUp(object):
+    def __init__(self):
+        main_frm = Toplevel()
+        main_frm.minsize(896, 507)
+        main_frm.wm_title("ABOUT SIMBA")
+        canvas = Canvas(main_frm,width=896,height=507,bg='black')
+        canvas.pack()
+        scriptdir = os.path.dirname(__file__)
+        img = PhotoImage(file=os.path.join(scriptdir,'About_me_050122_1.png'))
+        canvas.create_image(0,0,image=img,anchor='nw')
+        canvas.image = img
+
+class ConcatenatingVideosPopUp(object):
+    def __init__(self):
+        main_frm = Toplevel()
+        main_frm.minsize(300, 300)
+        main_frm.wm_title("CONCATENATE VIDEOS")
+        settings_frm = LabelFrame(main_frm, text='SETTINGS', font=('Helvetica', 12, 'bold'), pady=5, padx=5)
+        video_path_1 = FileSelect(settings_frm, "First video path: ", title='Select a video file')
+        video_path_2 = FileSelect(settings_frm, "Second video path: ", title='Select a video file')
+        resolutions =  ['Video 1', 'Video 2', 320, 640, 720, 1280, 1980]
+        resolution_dropdown = DropDownMenu(settings_frm, 'Resolution:', resolutions, '15')
+        resolution_dropdown.setChoices(resolutions[0])
+        horizontal = BooleanVar(value=False)
+        horizontal_radio_btn = Radiobutton(settings_frm, text="Horizontal concatenation", variable=horizontal, value=True)
+        vertical_radio_btn = Radiobutton(settings_frm, text="Vertical concatenation", variable=horizontal, value=False)
+        run_btn = Button(main_frm, text='RUN', font=('Helvetica', 12, 'bold'), command=lambda: video_concatenator(video_one_path=video_path_1.file_path,
+                                                                                      video_two_path=video_path_2.file_path,
+                                                                                      resolution=resolution_dropdown.getChoices(),
+                                                                                      horizontal=horizontal.get()))
+
+        settings_frm.grid(row=0, column=0, sticky=NW)
+        video_path_1.grid(row=0, column=0, sticky=NW)
+        video_path_2.grid(row=1, column=0, sticky=NW)
+        resolution_dropdown.grid(row=2,column=0,sticky=NW)
+        horizontal_radio_btn.grid(row=3,column=0,sticky=NW)
+        vertical_radio_btn.grid(row=4, column=0, sticky=NW)
+        run_btn.grid(row=1, column=0, sticky=NW)
+
+class VisualizePoseInFolderPopUp(object):
+    def __init__(self):
+        main_frm = Toplevel()
+        main_frm.minsize(350, 200)
+        main_frm.wm_title('Visualize pose-estimation')
+        settings_frame = LabelFrame(main_frm, text='SETTINGS', font=('Helvetica', 12, 'bold'), pady=5, padx=5)
+        self.input_folder = FolderSelect(settings_frame, 'Input directory (with csv/parquet files)', title='Select input folder')
+        self.output_folder = FolderSelect(settings_frame, 'Output directory (where your videos will be saved)', title='Select output folder')
+        self.circle_size = Entry_Box(settings_frame, 'Circle size', 0, validation='numeric')
+        run_btn = Button(main_frm, text='VISUALIZE POSE', font=('Helvetica', 12, 'bold'), command= lambda: self.run())
+
+        settings_frame.grid(row=0, sticky=W)
+        self.input_folder.grid(row=0, column=0, pady=10, sticky=W)
+        self.output_folder.grid(row=1, column=0, pady=10, sticky=W)
+        self.circle_size.grid(row=2, column=0, pady=10, sticky=W)
+        run_btn.grid(row=3, column=0, pady=10)
+
+
+    def run(self):
+        circle_size_int = self.circle_size.entry_get
+        input_folder = self.input_folder.folder_path
+        output_folder = self.output_folder.folder_path
+        if (input_folder == '') or (input_folder == 'No folder selected'):
+            print('SIMBA ERROR: Please select an input folder to continue')
+            raise ValueError('SIMBA ERROR: Please select an input folder to continue')
+        elif (output_folder == '') or (output_folder == 'No folder selected'):
+            print('SimBA ERROR: Please select an output folder to continue')
+            raise ValueError('SimBA ERROR: Please select an output folder to continue')
+        else:
+            create_video_from_dir(in_directory=input_folder, out_directory=output_folder, circle_size=int(circle_size_int))
+
+
+class DropTrackingDataPopUp(object):
+    def __init__(self):
+        self.main_frm = Toplevel()
+        self.main_frm.minsize(500, 800)
+        self.main_frm.wm_title('Drop body-parts in pose-estimation data')
+        self.main_frm.lift()
+        self.main_frm = Canvas(hxtScrollbar(self.main_frm))
+        self.main_frm.pack(fill="both", expand=True)
+        file_settings_frm = LabelFrame(self.main_frm, text='FILE SETTINGS', font=('Helvetica', 12, 'bold'), pady=5, padx=5)
+        self.data_folder_path = FolderSelect(file_settings_frm, 'Data Folder', lblwidth='20')
+        self.file_format = DropDownMenu(file_settings_frm, 'File Type', ['csv', 'h5'], '20')
+        self.pose_tool = DropDownMenu(file_settings_frm, 'Tracking tool', ['DLC', 'maDLC'], '20')
+        self.pose_tool.setChoices('DLC')
+        self.file_format.setChoices('csv')
+        self.bp_cnt = Entry_Box(file_settings_frm, '# body-parts to remove', '20', validation='numeric')
+        confirm_btn = Button(file_settings_frm, text='Confirm', command=lambda: self.confirm())
+        file_settings_frm.grid(row=0,sticky=NW)
+        self.data_folder_path.grid(row=0,sticky=W,columnspan=3)
+        self.pose_tool.grid(row=1,sticky=NW)
+        self.file_format.grid(row=2,sticky=NW)
+        self.bp_cnt.grid(row=3,sticky=NW)
+        confirm_btn.grid(row=3,column=1,sticky=NW)
+
+    def confirm(self):
+        if hasattr(self, 'bp_table'):
+            self.bp_table.destroy()
+        print(self.bp_cnt.entry_get)
+        self.keypoint_remover = KeypointRemover(data_folder=self.data_folder_path.folder_path, pose_tool=self.pose_tool.getChoices(), file_format=self.file_format.getChoices())
+        self.bp_table = LabelFrame(self.main_frm, text='REMOVE BODY-PARTS', font=('Helvetica', 12, 'bold'))
+        self.bp_table.grid(row=1, sticky=NW, pady=5)
+        self.animal_names_lst, self.drop_down_list = [], []
+        if self.pose_tool.getChoices() == 'DLC':
+            for bp_number in range(int(self.bp_cnt.entry_get)):
+                bp_drop_down = DropDownMenu(self.bp_table, 'Body-part {}:'.format(str(bp_number + 1)), self.keypoint_remover.body_part_names, '10')
+                bp_drop_down.setChoices(self.keypoint_remover.body_part_names[bp_number])
+                self.drop_down_list.append(bp_drop_down)
+                bp_drop_down.grid(row=bp_number, column=0, sticky=NW)
+        if self.pose_tool.getChoices()  == 'maDLC':
+            for bp_number in range(int(self.bp_cnt.entry_get)):
+                animal_drop_down = DropDownMenu(self.bp_table, 'Animal name:', self.keypoint_remover.animal_names, '10')
+                animal_drop_down.setChoices(self.keypoint_remover.animal_names[0])
+                self.animal_names_lst.append(animal_drop_down)
+                bp_drop_down = DropDownMenu(self.bp_table, 'Body-part {}:'.format(str(bp_number + 1)), self.keypoint_remover.body_part_names, '10')
+                bp_drop_down.setChoices(self.keypoint_remover.body_part_names[bp_number])
+                self.drop_down_list.append(bp_drop_down)
+                animal_drop_down.grid(row=bp_number, column=0, sticky=NW)
+                bp_drop_down.grid(row=bp_number, column=1, sticky=NW)
+
+        run_btn = Button(self.main_frm, text='RUN BODY-PART REMOVAL', command=lambda: self.run())
+        run_btn.grid(row=2, column=0, sticky=NW)
+
+    def run(self):
+        bp_to_remove_list, animal_names_list = [], []
+        for number, drop_down in enumerate(self.drop_down_list):
+            bp_to_remove_list.append(drop_down.getChoices())
+        if self.pose_tool == 'maDLC':
+            for number, drop_down in enumerate(self.animal_names_lst):
+                animal_names_list.append(drop_down.getChoices())
+        _ = self.keypoint_remover.run_bp_removal(bp_to_remove_list=bp_to_remove_list, animal_names=animal_names_list)
+
+class ConcatenatorPopUp(object):
+    def __init__(self,
+                 config_path: str or None):
+        self.config_path = config_path
+        self.icons_path = os.path.join(os.path.dirname(simba.__file__), 'assets', 'icons')
+        #self.icons_path = '/Users/simon/Desktop/simbapypi_dev/simba/assets/icons'
+        self.main_frm = Toplevel()
+        self.main_frm.minsize(500, 800)
+        self.main_frm.wm_title('MERGE (CONCATENATE) VIDEOS')
+        self.select_video_cnt_frm = LabelFrame(self.main_frm, text='VIDEOS #', pady=5, padx=5, font=("Helvetica", 12, 'bold'), fg='black')
+        self.select_video_cnt_dropdown = DropDownMenu(self.select_video_cnt_frm, 'VIDEOS #', list(range(2,21)), '15')
+        self.select_video_cnt_dropdown.setChoices(2)
+        self.select_video_cnt_btn = Button(self.select_video_cnt_frm, text='SELECT', command=lambda: self.populate_table())
+        self.select_video_cnt_frm.grid(row=0, column=0, sticky=NW)
+        self.select_video_cnt_dropdown.grid(row=0, column=0, sticky=NW)
+        self.select_video_cnt_btn.grid(row=0, column=1, sticky=NW)
+
+    def populate_table(self):
+        if hasattr(self, 'video_table_frm'):
+            self.video_table_frm.destroy()
+            self.join_type_frm.destroy()
+        self.video_table_frm = LabelFrame(self.main_frm, text='VIDEO PATHS', pady=5, padx=5, font=("Helvetica", 12, 'bold'), fg='black')
+        self.video_table_frm.grid(row=1, sticky=NW)
+        self.join_type_frm = LabelFrame(self.main_frm, text='JOIN TYPE', pady=5, padx=5, font=("Helvetica", 12, 'bold'),fg='black')
+        self.join_type_frm.grid(row=2, sticky=NW)
+        self.videos_dict = {}
+        for cnt in range(int(self.select_video_cnt_dropdown.getChoices())):
+            self.videos_dict[cnt] = FileSelect(self.video_table_frm, "Video {}: ".format(str(cnt+1)), title='Select a video file')
+            self.videos_dict[cnt].grid(row=cnt, column=0, sticky=NW)
+
+        self.join_type_var = StringVar()
+        self.icons_dict = {}
+        for file_cnt, file_path in enumerate(glob.glob(self.icons_path + '/*')):
+            _, file_name, _ = get_fn_ext(file_path)
+            self.icons_dict[file_name] = {}
+            # self.img1 = Image.open("pic1.png")
+            # self.pic1 = ImageTk.PhotoImage(self.img1)
+            #
+            #
+            self.icons_dict[file_name]['img'] = ImageTk.PhotoImage(Image.open(file_path))
+            self.icons_dict[file_name]['btn'] = Radiobutton(self.join_type_frm, text=file_name, variable=self.join_type_var, value=file_name)
+            self.icons_dict[file_name]['btn'].config(image=self.icons_dict[file_name]['img'])
+            self.icons_dict[file_name]['btn'].image = self.icons_dict[file_name]['img']
+            self.icons_dict[file_name]['btn'].grid(row=0, column=file_cnt, sticky=NW)
+        self.join_type_var.set(value='mosaic')
+
+        self.resolution_frm = LabelFrame(self.main_frm, text='RESOLUTION', pady=5, padx=5, font=("Helvetica", 12, 'bold'), fg='black')
+        self.resolution_width = DropDownMenu(self.resolution_frm, 'Resolution width', ['480', '640', '1280', '1920', '2560'], '15')
+        self.resolution_width.setChoices('640')
+        self.resolution_height = DropDownMenu(self.resolution_frm, 'Resolution height', ['480', '640', '1280', '1920', '2560'], '15')
+        self.resolution_height.setChoices('480')
+        self.resolution_frm.grid(row=3, column=0, sticky=NW)
+        self.resolution_width.grid(row=0, column=0, sticky=NW)
+        self.resolution_height.grid(row=1, column=0, sticky=NW)
+
+        run_btn = Button(self.main_frm, text='RUN', command=lambda: self.run())
+        run_btn.grid(row=4, column=0, sticky=NW)
+
+    def run(self):
+        videos_info = {}
+        for cnt, (video_name, video_data) in enumerate(self.videos_dict.items()):
+            _ = get_video_meta_data(video_path=video_data.file_path)
+            videos_info['Video {}'.format(str(cnt+1))] = video_data.file_path
+
+        if (len(videos_info.keys()) < 3) & (self.join_type_var.get() == 'mixed_mosaic'):
+            print('SIMBA ERROR: if using the mixed mosaic join type, please tick check-boxes for at leasr three video types.')
+            raise ValueError()
+
+        if (len(videos_info.keys()) < 3) & (self.join_type_var.get() == 'mosaic'):
+            self.join_type_var.set(value='vertical')
+
+
+        _ = FrameMergererFFmpeg(config_path=self.config_path,
+                                frame_types=videos_info,
+                                video_height=int(self.resolution_height.getChoices()),
+                                video_width=int(self.resolution_width.getChoices()),
+                                concat_type=self.join_type_var.get())
+
+class SetMachineModelParameters(object):
+    def __init__(self,
+                 config_path: str):
+
+        self.main_frm = Toplevel()
+        self.main_frm.minsize(200, 200)
+        self.main_frm.wm_title("SET MODEL PARAMETERS")
+        self.main_frm.lift()
+        self.main_frm = Canvas(hxtScrollbar(self.main_frm))
+        self.main_frm.pack(fill="both", expand=True)
+        self.config, self.config_path = read_config_file(ini_path=config_path), config_path
+        self.clf_cnt = read_config_entry(self.config, 'SML settings', 'no_targets', 'int')
+        self.clf_names = list(get_all_clf_names(config=self.config,target_cnt=self.clf_cnt))
+
+        self.clf_table_frm = LabelFrame(self.main_frm)
+        Label(self.clf_table_frm, text='CLASSIFIER', font=("Helvetica", 12, 'bold')).grid(row=0, column=0)
+        Label(self.clf_table_frm, text='MODEL PATH (.SAV)', font=("Helvetica", 12, 'bold')).grid(row=0, column=1)
+        Label(self.clf_table_frm, text='THRESHOLD', font=("Helvetica", 12, 'bold')).grid(row=0, column=2)
+        Label(self.clf_table_frm, text='MINIMUM BOUT LENGTH (MS)', font=("Helvetica", 12, 'bold')).grid(row=0, column=3)
+        self.clf_data = {}
+        for clf_cnt, clf_name in enumerate(self.clf_names):
+            self.clf_data[clf_name] = {}
+            Label(self.clf_table_frm, text=clf_name, font=("Helvetica", 14)).grid(row=clf_cnt+1, column=0, sticky=NW)
+            self.clf_data[clf_name]['path'] = FileSelect(self.clf_table_frm, title='Select model (.sav) file')
+            self.clf_data[clf_name]['threshold'] = Entry_Box(self.clf_table_frm, '', '15')
+            self.clf_data[clf_name]['min_bout'] = Entry_Box(self.clf_table_frm, '', '15')
+            self.clf_data[clf_name]['path'].grid(row=clf_cnt+1, column=1, sticky=NW)
+            self.clf_data[clf_name]['threshold'].grid(row=clf_cnt+1, column=2, sticky=NW)
+            self.clf_data[clf_name]['min_bout'].grid(row=clf_cnt + 1, column=3, sticky=NW)
+
+        set_btn = Button(self.main_frm, text='SET MODEL(S)', font=("Helvetica",12,'bold'),fg='red', command = lambda:self.set())
+        self.clf_table_frm.grid(row=0,sticky=W,pady=5,padx=5)
+        set_btn.grid(row=1,pady=10)
+
+    def set(self):
+        for model_name, model_settings in self.clf_data.items():
+            check_file_exist_and_readable(model_settings['path'].file_path)
+            check_float(name='Classifier {} threshhold'.format(model_name), value=model_settings['threshold'].entry_get, max_value=1.0, min_value=0.0)
+            check_int(name='Classifier {} minimum bout'.format(model_name), value=model_settings['min_bout'].entry_get, min_value=0.0)
+
+        for cnt, (model_name, model_settings) in enumerate(self.clf_data.items()):
+            self.config.set('SML settings', 'model_path_{}'.format(str(cnt+1)), model_settings['path'].file_path)
+            self.config.set('threshold_settings', 'threshold_{}'.format(str(cnt+1)), model_settings['threshold'].entry_get)
+            self.config.set('Minimum_bout_lengths', 'min_bout_{}'.format(str(cnt+1)), model_settings['min_bout'].entry_get)
+
+        with open(self.config_path, 'w') as f:
+            self.config.write(f)
+
+        print('SIMBA COMPLETE: Model paths/settings saved in project_config.ini')
+
+class OutlierSettingsPopUp(object):
+    def __init__(self,
+                 config_path: str):
+
+        self.config_path, self.config = config_path, read_config_file(ini_path=config_path)
+        self.animal_cnt = read_config_entry(config=self.config, section='General settings',option='animal_no',data_type='int')
+        self.main_frm = Toplevel()
+        self.main_frm.minsize(400, 400)
+        self.main_frm.wm_title("Outlier Settings")
+        self.main_frm.lift()
+        self.main_frm = Canvas(hxtScrollbar(self.main_frm))
+        self.main_frm.pack(fill="both", expand=True)
+
+        x_cols, y_cols, pcols = getBpNames(self.config_path)
+        multi_animal_status, animal_id_list = check_multi_animal_status(self.config, self.animal_cnt)
+        self.animal_bp_dict = create_body_part_dictionary(multi_animal_status, animal_id_list, self.animal_cnt, x_cols, y_cols, [], [])
+        self.animal_bps = {}
+        for animal_name, animal_data in self.animal_bp_dict.items(): self.animal_bps[animal_name] = [x[:-2] for x in animal_data['X_bps']]
+        self.location_correction_frm = LabelFrame(self.main_frm, text='LOCATION CORRECTION', font=('Times', 12, 'bold'), pady=5, padx=5)
+
+        bp_entry_cnt, self.criterion_dropdowns = 0, {}
+        for animal_cnt, animal_name in enumerate(self.animal_bp_dict.keys()):
+            self.criterion_dropdowns[animal_name] = {}
+            self.criterion_dropdowns[animal_name]['location_bp_1'] = DropDownMenu(self.location_correction_frm, 'Choose {} body part 1:'.format(animal_name), self.animal_bps[animal_name], '30')
+            self.criterion_dropdowns[animal_name]['location_bp_2'] = DropDownMenu(self.location_correction_frm, 'Choose {} body part 2:'.format(animal_name), self.animal_bps[animal_name], '30')
+            self.criterion_dropdowns[animal_name]['location_bp_1'].setChoices(self.animal_bps[animal_name][0])
+            self.criterion_dropdowns[animal_name]['location_bp_2'].setChoices(self.animal_bps[animal_name][1])
+            self.criterion_dropdowns[animal_name]['location_bp_1'].grid(row=bp_entry_cnt, column=0, sticky=NW)
+            bp_entry_cnt+=1
+            self.criterion_dropdowns[animal_name]['location_bp_2'].grid(row=bp_entry_cnt, column=0, sticky=NW)
+            bp_entry_cnt+=1
+        self.location_criterion = Entry_Box(self.location_correction_frm, 'Location criterion: ', '15')
+        self.location_criterion.grid(row=bp_entry_cnt, column=0, sticky=NW)
+        self.location_correction_frm.grid(row=0, column=0, sticky=NW)
+
+        self.movement_correction_frm = LabelFrame(self.main_frm, text='MOVEMENT CORRECTION', font=('Times', 12, 'bold'), pady=5, padx=5)
+        bp_entry_cnt = 0
+        for animal_cnt, animal_name in enumerate(self.animal_bp_dict.keys()):
+            self.criterion_dropdowns[animal_name]['movement_bp_1'] = DropDownMenu(self.movement_correction_frm, 'Choose {} body part 1:'.format(animal_name), self.animal_bps[animal_name], '30')
+            self.criterion_dropdowns[animal_name]['movement_bp_2'] = DropDownMenu(self.movement_correction_frm, 'Choose {} body part 2:'.format(animal_name), self.animal_bps[animal_name], '30')
+            self.criterion_dropdowns[animal_name]['movement_bp_1'].setChoices(self.animal_bps[animal_name][0])
+            self.criterion_dropdowns[animal_name]['movement_bp_2'].setChoices(self.animal_bps[animal_name][1])
+            self.criterion_dropdowns[animal_name]['movement_bp_1'].grid(row=bp_entry_cnt, column=0, sticky=NW)
+            bp_entry_cnt+=1
+            self.criterion_dropdowns[animal_name]['movement_bp_2'].grid(row=bp_entry_cnt, column=0, sticky=NW)
+            bp_entry_cnt+=1
+        self.movement_criterion = Entry_Box(self.movement_correction_frm, 'Location criterion: ', '15')
+        self.movement_criterion.grid(row=bp_entry_cnt, column=0, sticky=NW)
+        self.movement_correction_frm.grid(row=1, column=0, sticky=NW)
+
+        agg_type_frm = LabelFrame(self.main_frm, text='AGGREGATION METHOD', font=('Times', 12, 'bold'), pady=5, padx=5)
+        self.agg_type_dropdown = DropDownMenu(agg_type_frm, 'Aggregation method:', ['mean', 'median'], '15')
+        self.agg_type_dropdown.setChoices('median')
+        self.agg_type_dropdown.grid(row=0, column=0, sticky=NW)
+        agg_type_frm.grid(row=2, column=0, sticky=NW)
+
+        run_btn = Button(self.main_frm, text='CONFIRM', font=('Helvetica', 12, 'bold'), fg='red', command=lambda: self.run())
+        run_btn.grid(row=3, column=0, sticky=NW)
+
+    def run(self):
+        check_float(name='LOCATION CRITERION', value=self.location_criterion.entry_get, min_value=0.0)
+        check_float(name='MOVEMENT CRITERION', value=self.movement_criterion.entry_get, min_value=0.0)
+        self.config.set('Outlier settings', 'movement_criterion', str(self.movement_criterion.entry_get))
+        self.config.set('Outlier settings', 'location_criterion', str(self.location_criterion.entry_get))
+        self.config.set('Outlier settings', 'mean_or_median', str(self.agg_type_dropdown.getChoices()))
+        for animal_cnt, animal_name in enumerate(self.animal_bp_dict.keys()):
+            self.config.set('Outlier settings', 'movement_bodyPart1_{}'.format(animal_name), self.criterion_dropdowns[animal_name]['movement_bp_1'].getChoices())
+            self.config.set('Outlier settings', 'movement_bodyPart2_{}'.format(animal_name), self.criterion_dropdowns[animal_name]['movement_bp_2'].getChoices())
+            self.config.set('Outlier settings', 'location_bodyPart1_{}'.format(animal_name), self.criterion_dropdowns[animal_name]['location_bp_1'].getChoices())
+            self.config.set('Outlier settings', 'location_bodyPart2_{}'.format(animal_name), self.criterion_dropdowns[animal_name]['location_bp_2'].getChoices())
+        with open(self.config_path, 'w') as f:
+            self.config.write(f)
+
+        print('SIMBA COMPLETE: Outlier correction settings updated in the project_config.ini')
+
+class RemoveAClassifierPopUp(object):
+    def __init__(self,
+                 config_path: str):
+        self.main_frm = Toplevel()
+        self.main_frm.minsize(200, 200)
+        self.main_frm.wm_title("Warning: Remove classifier(s) settings")
+        self.config, self.config_path = read_config_file(ini_path=config_path), config_path
+        self.target_cnt = read_config_entry(self.config, 'SML settings', 'no_targets', 'int')
+        self.clf_names = list(get_all_clf_names(config=self.config, target_cnt=self.target_cnt))
+
+        self.remove_clf_frm = LabelFrame(self.main_frm, text='SELECT A CLASSIFIER TO REMOVE')
+        self.clf_dropdown = DropDownMenu(self.remove_clf_frm, 'Classifier', self.clf_names, '12')
+        self.clf_dropdown.setChoices(self.clf_names[0])
+
+        run_btn = Button(self.main_frm,text='REMOVE CLASSIFIER',command=lambda:self.run())
+        self.remove_clf_frm.grid(row=0,sticky=W)
+        self.clf_dropdown.grid(row=0,sticky=W)
+        run_btn.grid(row=1,pady=10)
+
+    def run(self):
+        for i in range(len(self.clf_names)):
+            self.config.remove_option('SML settings', 'model_path_{}'.format(str(i+1)))
+            self.config.remove_option('SML settings', 'target_name_{}'.format(str(i+1)))
+            self.config.remove_option('threshold_settings', 'threshold_{}'.format(str(i+1)))
+            self.config.remove_option('Minimum_bout_lengths', 'min_bout_{}'.format(str(i+1)))
+        self.clf_names.remove(self.clf_dropdown.getChoices())
+        self.config.set('SML settings', 'no_targets', str(len(self.clf_names)))
+
+        for clf_cnt, clf_name in enumerate(self.clf_names):
+            self.config.set('SML settings', 'model_path_{}'.format(str(clf_cnt+1)), '')
+            self.config.set('SML settings', 'target_name_{}'.format(str(clf_cnt+1)), clf_name)
+            self.config.set('threshold_settings', 'threshold_{}'.format(str(clf_cnt+1)), 'None')
+            self.config.set('Minimum_bout_lengths', 'min_bout_{}'.format(str(clf_cnt+1)), 'None')
+
+        with open(self.config_path, 'w') as f:
+            self.config.write(f)
+
+        print('SIMBA COMPLETE: {} classifier removed from SimBA project.'.format(self.clf_dropdown.getChoices()))
+#
+#
+# class GetPxPerMmPopUp(object):
+#     def __init__(self):
+#         self.main_frm = Toplevel()
+#         self.main_frm.minsize(200, 200)
+#         self.main_frm.wm_title("GET PIXELS PER MILLIMETER IN VIDEO")
+#
+#
+#
+#
+# class Button_getcoord(Frame):
+#
+#     def __init__(self,parent=None,filename=[],knownmm=[],ppmlist = None,**kw): #set to list and use range i in list to call each elements in list
+#
+#         Frame.__init__(self, master=parent, **kw)
+#         self.entPath = []
+#         self.ppm_list = []
+#         self.ppmvar = []
+#         self.filename = filename
+#         labelgetcoord =Label(self,text='Get coord')
+#         labelgetcoord.grid(row=0,pady=6)
+#
+#         labelppm =Label(self,text='Pixels/mm')
+#         labelppm.grid(row=0,column=1)
+#
+#         for i in range(len(filename)-1):
+#             self.entPath.append(Button(self,text='Video'+str(i+1),command =lambda i=i :self.getcoord_forbutton(filename[i+1],knownmm[i+1],i)))
+#             self.entPath[i].grid(row=i+1)
+#             self.ppmvars=IntVar()
+#             self.ppmvar.append(self.ppmvars)
+#             self.ppm_list.append(Entry(self,textvariable=self.ppmvar[i]))
+#             self.ppm_list[i].grid(row=i+1,column=1)
+#         if ppmlist != None:
+#             try:
+#                 for i in range(len(self.ppmvar)):
+#                     self.ppmvar[i].set(ppmlist[i])
+#             except IndexError:
+#                 pass
\ No newline at end of file
