@@ -5,6 +5,7 @@ from configparser import ConfigParser
 import cv2
 from simba.roi_tools.ROI_image import ROI_image_class
 import pandas as pd
+from simba.tkinter_functions import hxtScrollbar
 from simba.roi_tools.ROI_move_shape import update_all_tags, move_edge
 from simba.roi_tools.ROI_multiply import create_emty_df
 from simba.roi_tools.ROI_size_calculations import rectangle_size_calc, circle_size_calc, polygon_size_calc
@@ -71,6 +72,10 @@ class ROI_definitions:
         self.default_top_left_x = self.screen_width - self.master_win_w
         self.master.geometry('%dx%d+%d+%d' % (self.master_win_w, self.master_win_h, self.default_top_left_x, 0))
         self.master.wm_title("Region of Interest Settings")
+
+
+
+
         self.shape_thickness_list = list(range(1, 26))
         self.ear_tag_size_list = list(range(1, 26))
         self.select_color = 'red'
@@ -103,6 +108,9 @@ class ROI_definitions:
                                    'Purple': (255, 51, 153),
                                    'Cyan': (255, 255, 102)}
         self.window_menus()
+        self.master.lift()
+        self.master = Canvas(hxtScrollbar(self.master))
+        self.master.pack(fill="both", expand=True)
         self.show_video_info()
         self.select_img()
         self.apply_from_other_videos_menu()
