@@ -1,4 +1,8 @@
-from simba.read_config_unit_tests import read_config_entry, read_config_file, check_file_exist_and_readable
+from simba.read_config_unit_tests import (read_config_entry,
+                                          read_config_file,
+                                          check_file_exist_and_readable,
+                                          check_float,
+                                          check_int)
 from simba.misc_tools import check_multi_animal_status, find_video_of_file, get_video_meta_data
 from simba.rw_dfs import read_df
 from simba.misc_tools import create_single_color_lst
@@ -21,6 +25,8 @@ class PlotSklearnResults(object):
     ----------
     config_path: str
         path to SimBA project config file in Configparser format
+    rotate: bool
+        If True, the output video will be rotated 90 degrees from the input.
     video_setting: bool
         If True, SimBA will create compressed videos.
     frame_setting: bool
@@ -34,7 +40,7 @@ class PlotSklearnResults(object):
 
     Examples
     ----------
-    >>> clf_plotter = PlotSklearnResults(config_path='MyProjectConfig', video_setting=True, frame_setting=False, video_file_path='VideoPath')
+    >>> clf_plotter = PlotSklearnResults(config_path='MyProjectConfig', video_setting=True, frame_setting=False, rotate=False, video_file_path='VideoPath')
     >>> clf_plotter.initialize_visualizations()
     """
 
@@ -42,12 +48,15 @@ class PlotSklearnResults(object):
                  config_path: str,
                  video_setting: bool,
                  frame_setting: bool,
+                 text_settings: dict or bool,
+                 rotate: False,
                  video_file_path=None,
                  print_timers=True):
 
         if (not video_setting) and (not frame_setting):
+            print('SIMBA ERROR: Please choose to create a video and/or frames. SimBA found that you ticked neither video and/or frames')
             raise ValueError('SIMBA ERROR: Please choose to create a video and/or frames. SimBA found that you ticked neither video and/or frames')
-        self.video_file_path, self.print_timers = video_file_path, print_timers
+        self.video_file_path, self.print_timers, self.text_settings = video_file_path, print_timers, text_settings
         self.config = read_config_file(config_path)
         self.video_setting, self.frame_setting = video_setting, frame_setting
         self.project_path = read_config_entry(self.config, 'General settings', 'project_path', data_type='folder_path')
@@ -71,10 +80,28 @@ class PlotSklearnResults(object):
         self.animal_bp_dict = create_body_part_dictionary(self.multi_animal_status, self.multi_animal_id_lst, self.no_animals, self.x_cols, self.y_cols, [], self.color_lst_of_lst)
         self.fourcc = cv2.VideoWriter_fourcc(*'mp4v')
         self.font = cv2.FONT_HERSHEY_COMPLEX
+        self.rotate = rotate
         self.a = np.deg2rad(90)
-
         print('Processing {} videos...'.format(str(len(self.files_found))))
 
+    def __get_print_settings(self):
+        if self.text_settings is False:
+            self.space_scale, self.radius_scale, self.res_scale, self.font_scale = 60, 12, 1500, 1.1
+            self.max_dim = max(self.video_meta_data['width'], self.video_meta_data['height'])
+            self.circle_scale = int(self.radius_scale / (self.res_scale / self.max_dim))
+            self.font_size = float(self.font_scale / (self.res_scale / self.max_dim))
+            self.spacing_scale = int(self.space_scale / (self.res_scale / self.max_dim))
+            self.text_thickness = 2
+        else:
+            check_float(name='ERROR: TEXT SIZE', value=self.text_settings['font_size'])
+            check_int(name='ERROR: SPACE SIZE', value=self.text_settings['space_size'])
+            check_int(name='ERROR: TEXT THICKNESS', value=self.text_settings['text_thickness'])
+            check_int(name='ERROR: CIRCLE SIZE', value=self.text_settings['circle_size'])
+            self.font_size = float(self.text_settings['font_size'])
+            self.spacing_scale = int(self.text_settings['space_size'])
+            self.text_thickness = int(self.text_settings['text_thickness'])
+            self.circle_scale = int(self.text_settings['circle_size'])
+
     def create_visualizations(self):
         _, self.video_name, _ = get_fn_ext(self.file_path)
         self.data_df = read_df(self.file_path, self.file_type).reset_index(drop=True)
@@ -87,17 +114,10 @@ class PlotSklearnResults(object):
         if self.frame_setting:
             self.video_frame_dir = os.path.join(self.save_folder, self.video_name)
             if not os.path.exists(self.video_frame_dir): os.makedirs(self.video_frame_dir)
-        if self.video_meta_data['height'] < self.video_meta_data['width']:
+        if self.rotate:
             self.video_meta_data['height'], self.video_meta_data['width'] = width, height
-            self.rotation_flag = True
-        else:
-            self.rotation_flag = False
         self.writer = cv2.VideoWriter(self.save_path, self.fourcc, self.fps, (self.video_meta_data['width'], self.video_meta_data['height']))
-        self.space_scale, self.radius_scale, self.res_scale, self.font_scale = 60, 12, 1500, 1.1
-        self.max_dim = max(self.video_meta_data['width'], self.video_meta_data['height'])
-        self.circle_scale = int(self.radius_scale / (self.res_scale / self.max_dim))
-        self.font_size = float(self.font_scale / (self.res_scale / self.max_dim))
-        self.spacing_scale = int(self.space_scale / (self.res_scale / self.max_dim))
+        self.__get_print_settings()
         self.video_model_dict = deepcopy(self.model_dict)
         for model in self.video_model_dict:
             self.video_model_dict[model]['time'] = 0
@@ -126,27 +146,27 @@ class PlotSklearnResults(object):
                             self.id_flag_cords[animal_name] = (int(bp_cords[x_bp]), int(bp_cords[y_bp]))
 
                     for animal_name, animal_cords in self.id_flag_cords.items():
-                        cv2.putText(self.frame, animal_name, animal_cords, self.font, self.font_scale,
+                        cv2.putText(self.frame, animal_name, animal_cords, self.font, self.font_size,
                                     self.animal_bp_dict[animal_name]['colors'][0], 2)
 
-                    if self.rotation_flag:
+                    if self.rotate:
                         self.frame = np.array(Image.fromarray(self.frame).rotate(90, Image.BICUBIC, expand=True))
                     if self.print_timers:
-                        cv2.putText(self.frame, str('Timers'), (10, ((self.video_meta_data['height'] - self.video_meta_data['height']) + self.spacing_scale)), self.font, self.font_scale, (0, 255, 0), 2)
+                        cv2.putText(self.frame, str('Timers'), (10, ((self.video_meta_data['height'] - self.video_meta_data['height']) + self.spacing_scale)), self.font, self.font_size, (0, 255, 0), self.text_thickness)
                     self.add_spacer = 2
                     for model_no, model_info in self.video_model_dict.items():
                         frame_results = self.data_df.loc[row_n, model_info['model_name']]
                         self.video_model_dict[model_no]['frame_results'] = frame_results
                         self.video_model_dict[model_no]['time'] += frame_results / self.fps
                         if self.print_timers:
-                            cv2.putText(self.frame, model_info['model_name'] + ' ' + str(round(self.video_model_dict[model_no]['time'], 2)) + str('s'), (10, (self.video_meta_data['height'] - self.video_meta_data['height']) + self.spacing_scale * self.add_spacer), self.font, self.font_scale, (255, 0, 0), 2)
+                            cv2.putText(self.frame, model_info['model_name'] + ' ' + str(round(self.video_model_dict[model_no]['time'], 2)) + str('s'), (10, (self.video_meta_data['height'] - self.video_meta_data['height']) + self.spacing_scale * self.add_spacer), self.font, self.font_size, (255, 0, 0), self.text_thickness)
                             self.add_spacer += 1
-                    cv2.putText(self.frame, str('Ensemble prediction'), (10, (self.video_meta_data['height'] - self.video_meta_data['height']) + self.spacing_scale * self.add_spacer), self.font, self.font_scale, (0, 255, 0), 2)
+                    cv2.putText(self.frame, str('Ensemble prediction'), (10, (self.video_meta_data['height'] - self.video_meta_data['height']) + self.spacing_scale * self.add_spacer), self.font, self.font_size, (0, 255, 0), self.text_thickness)
                     self.add_spacer += 1
 
                     for model_cnt, model_info in self.video_model_dict.items():
                         if self.video_model_dict[model_cnt]['frame_results'] == 1:
-                            cv2.putText(self.frame, model_info['model_name'], (10, ( self.video_meta_data['height'] - self.video_meta_data['height']) + self.spacing_scale * self.add_spacer), self.font, self.font_scale, self.clf_colors[model_cnt], 2)
+                            cv2.putText(self.frame, model_info['model_name'], (10, ( self.video_meta_data['height'] - self.video_meta_data['height']) + self.spacing_scale * self.add_spacer), self.font, self.font_size, self.clf_colors[model_cnt], self.text_thickness)
                             self.add_spacer += 1
                     if self.video_setting:
                         self.writer.write(self.frame)
@@ -181,6 +201,7 @@ class PlotSklearnResults(object):
             self.file_cnt, file_path = 0, self.video_file_path
             _, file_name, _ = get_fn_ext(file_path)
             self.file_path = os.path.join(self.data_folder, file_name + '.' + self.file_type)
+            self.files_found = [self.file_path]
             check_file_exist_and_readable(self.file_path)
             self.create_visualizations()
 
