@@ -8,46 +8,9 @@ from simba.misc_tools import get_fn_ext, get_video_meta_data
 from simba.read_config_unit_tests import read_config_file, read_config_entry
 from simba.features_scripts.unit_tests import read_video_info_csv, read_video_info
 from simba.get_coordinates_tools_v2 import get_coordinates_nilsson
+from simba.tkinter_functions import onMousewheel, hxtScrollbar, unbindToMousewheel, onFrameConfigure, bindToMousewheel
 import collections
 
-def onMousewheel(event, canvas):
-    try:
-        scrollSpeed = event.delta
-        if platform.system() == 'Darwin':
-            scrollSpeed = event.delta
-        elif platform.system() == 'Windows':
-            scrollSpeed = int(event.delta / 120)
-        canvas.yview_scroll(-1 * (scrollSpeed), "units")
-    except:
-        pass
-
-def _bindToMousewheel(event, canvas):
-    canvas.bind_all("<MouseWheel>", lambda event: _onMousewheel(event, canvas))
-
-def _unbindToMousewheel(event, canvas):
-    canvas.unbind_all("<MouseWheel>")
-
-def _onFrameConfigure(canvas):
-    canvas.configure(scrollregion=canvas.bbox("all"))
-
-def _hxtScrollbar(master):
-    bg = master.cget("background")
-    acanvas = Canvas(master, borderwidth=0, background=bg)
-    frame = Frame(acanvas, background=bg)
-    vsb = Scrollbar(master, orient="vertical", command=acanvas.yview)
-    vsb2 = Scrollbar(master, orient='horizontal', command=acanvas.xview)
-    acanvas.configure(yscrollcommand=vsb.set)
-    acanvas.configure(xscrollcommand=vsb2.set)
-    vsb.pack(side="right", fill="y")
-    vsb2.pack(side="bottom", fill="x")
-    acanvas.pack(side="left", fill="both", expand=True)
-
-    acanvas.create_window((10, 10), window=frame, anchor="nw")
-    acanvas.bind("<Configure>", lambda event, canvas=acanvas: _onFrameConfigure(acanvas))
-    acanvas.bind('<Enter>', lambda event: _bindToMousewheel(event, acanvas))
-    acanvas.bind('<Leave>', lambda event: _unbindToMousewheel(event,acanvas))
-    return frame
-
 class VideoInfoTable(object):
     """
     Class for creating Tkinter GUI video meta data table. Allows users to modify resolutions, fps, and pixels-per-mm
@@ -163,7 +126,7 @@ class VideoInfoTable(object):
         self.main_frm = Toplevel()
         self.main_frm.minsize(1550, 800)
         self.main_frm.wm_title("Video Info")
-        self.main_frm = Canvas(_hxtScrollbar(self.main_frm))
+        self.main_frm = Canvas(hxtScrollbar(self.main_frm))
         self.main_frm.pack(expand=True,fill=BOTH)
         self.instructions_frm = LabelFrame(self.main_frm, text='INSTRUCTIONS', font=('Helvetica', 15, 'bold'))
         self.intructions_label_1 = Label(self.instructions_frm, text='1. Enter the known distance (millimeters) in the "DISTANCE IN MM" column. Consider using the "autopopulate" entry box in the main project window you have a lot of videos.', font=('Helvetica', 15))
